﻿//
// Copyright (C) 2009 Chris Dziemborowicz
//
// This file is part of Orzeszek Ratings.
//
// Orzeszek Ratings is free software: you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation, either version 3 of the
// License, or (at your option) any later version.
//
// Orzeszek Ratings is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Interop;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace OrzeszekRatings
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		private Thread systemUpdaterThread;
		private Thread interfaceUpdaterThread;

		public MainWindow()
		{
			InitializeComponent();

			StatusBarManager.StatusBar = StatusBar;

			StartSyncControl.SettingsControl = SettingsControl;

			StartSyncControl.SyncPlayCountsCheckBox.IsChecked = Settings.Default.SyncPlayCounts;
			StartSyncControl.SyncRatingsCheckBox.IsChecked = Settings.Default.SyncRatings;

			SettingsControl.ForceRatingsUpdateCheckBox.IsChecked = Settings.Default.ForceRatingsUpdate;
			SettingsControl.OnlySyncHighestPlayCountCheckBox.IsChecked = Settings.Default.OnlySyncHighestPlayCount;
			SettingsControl.AggregateMovePlayCountCheckBox.IsChecked = Settings.Default.OnlySyncHighestPlayCount ? false : Settings.Default.AggregateMovePlayCount;

			if (Settings.Default.WindowSettingsSaved)
				try
				{
					Height = Settings.Default.WindowHeight;
					Width = Settings.Default.WindowWidth;
					Left = Settings.Default.WindowLeft;
					Top = Settings.Default.WindowTop;
					WindowState = Settings.Default.WindowState;
				}
				catch (Exception)
				{
				}
		}

		private void Window_Loaded(object sender, RoutedEventArgs e)
		{
			SyncProgressControl.InteropHelper = new WindowInteropHelper(this);

			interfaceUpdaterThread = new Thread(new ThreadStart(ProcessInterfaceUpdates));
			interfaceUpdaterThread.Start();

			systemUpdaterThread = new Thread(new ThreadStart(ProcessSystemUpdates));
			systemUpdaterThread.Start();
		}

		private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			if (SyncProgressControl.SyncInfo != null)
				e.Cancel = true;
			else
				try
				{
					Settings.Default.SyncPlayCounts = StartSyncControl.SyncPlayCountsCheckBox.IsChecked.Value;
					Settings.Default.SyncRatings = StartSyncControl.SyncRatingsCheckBox.IsChecked.Value;

					Settings.Default.ForceRatingsUpdate = SettingsControl.ForceRatingsUpdateCheckBox.IsChecked.Value;
					Settings.Default.OnlySyncHighestPlayCount = SettingsControl.OnlySyncHighestPlayCountCheckBox.IsChecked.Value;
					Settings.Default.AggregateMovePlayCount = SettingsControl.AggregateMovePlayCountCheckBox.IsChecked.Value;

					Settings.Default.WindowHeight = RestoreBounds.Height;
					Settings.Default.WindowWidth = RestoreBounds.Width;
					Settings.Default.WindowLeft = RestoreBounds.Left;
					Settings.Default.WindowTop = RestoreBounds.Top;
					Settings.Default.WindowState = WindowState;
					Settings.Default.WindowSettingsSaved = true;

					Settings.Default.Save();
				}
				catch (Exception)
				{
				}
		}

		private void Window_Closed(object sender, EventArgs e)
		{
			if (interfaceUpdaterThread != null)
				interfaceUpdaterThread.Abort();

			SyncProgressControl.Close();
		}

		private void StartSyncControl_SyncRequested(object sender, SyncInfoEventArgs e)
		{
			SyncProgressControl.StartSync(e.SyncInfo);
		}

		private void SyncProgressControl_Done(object sender, EventArgs e)
		{
			SyncProgressControl.AnimateHide();
			StartSyncControl.Reset();
			StartSyncControl.AnimateShow();
		}

		private void SyncProgressControl_CloseRequested(object sender, EventArgs e)
		{
			Close();
		}

		private void ProcessInterfaceUpdates()
		{
			while (true)
			{
				Dispatcher.BeginInvoke(new Action(UpdateInterface));
				Thread.Sleep(1000);
			}
		}

		private void UpdateInterface()
		{
			SyncProgressControl.UpdateInterface();
			StatusBar.UpdateInterface();
		}

		private void ProcessSystemUpdates()
		{
			UpdateInfo ui = UpdateUtility.GetLatestUpdate();
			if (ui != null)
				StatusBarManager.AddMessage(string.IsNullOrEmpty(ui.Message) ? string.Format("Orzeszek Ratings {0} is now available.", ui.LatestVersion) : ui.Message, MessageType.Info, MessageLength.Infinite, "Download", ui.UpdateUrl);
		}
	}
}